/*
 * Copyright 2010-2011 NEC Infrontia Corp, Inc.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <hwconfig.h>
#include <pci.h>
#include <i2c.h>
#include <asm/processor.h>
#include <asm/mmu.h>
#include <asm/cache.h>
#include <asm/immap_85xx.h>
#include <asm/fsl_pci.h>
#include <asm/fsl_ddr_sdram.h>
#include <asm/io.h>
#include <asm/fsl_law.h>
#include <asm/fsl_lbc.h>
#include <asm/mp.h>
#include <miiphy.h>
#include <libfdt.h>
#include <fdt_support.h>
#include <tsec.h>
#include <ioports.h>
#include <asm/fsl_serdes.h>
#include <netdev.h>

extern void cpld_check_rev(void);
extern void board_cpld_init(void);
extern unsigned char cpld_testmode(void);

/*
* ------	-	---
* GPIO05	O	SEC_IC_SDA						
* GPIO06	O	SEC_IC_SCL						
* GPIO07	O	Reset to DDR3 device			0 = reset  1 = out of reset
* -----		-	---								---
* -----		-	---								---
* GPIO10	O	USB_LED							0 = on	   1 = off
* GPIO11	I	RMII/RGMII SELECT FOR IPL[B/E]	0 = RGMII  1 = RMII
* GPIO12	I	Detect VOIPDB Card				0 = none   1 = detect
* GPIO13	O	Reset to 88E151x device			0 = reset  1 = out of reset
* GPIO14	O	Reset to VOIPDB CPLD device		0 = reset  1 = out of reset
* GPIO15	O	Reset to USB3300 devices		1 = reset  0 = out of reset
*/
#define GPIO05	0x04000000
#define GPIO06	0x02000000
#define GPIO07	0x01000000
#define GPIO10	0x00200000
#define GPIO11	0x00100000
#define GPIO12	0x00080000
#define GPIO13	0x00040000
#define GPIO14	0x00020000
#define GPIO15	0x00010000
#define GPIO_DIR	(GPIO05|GPIO06|GPIO07|GPIO10|GPIO13|GPIO14|GPIO15)

/*
 board_type
 ret: 0 ... WS1 - CS2:CPLD
      1 ... WS2 - CS2:FPGA
 */
u8 board_revision( void ) {
    /*LAD[0:3] ... Board Rev */
	ccsr_gur_t *gur = (void *)(CONFIG_SYS_MPC85xx_GUTS_ADDR);
	return (((in_be32(&gur->gpporcr) & 0xF0000000) >> 28) & 0xf);
}

#define SYSCLK_65	65536000
#define SYSCLK_66	66666666
unsigned long get_board_sys_clk(ulong dummy)
{
	if( board_revision() >= 0x1 )
		return SYSCLK_65;
	return SYSCLK_66;
}
#define DDRCLK_81	81920000
#define DDRCLK_83	83333333
unsigned long get_board_ddr_clk(ulong dummy)
{
	if( board_revision() >= 0x1 )
		return DDRCLK_81;
	return DDRCLK_83;
}

void board_gpio_init(void)
{
	ccsr_gpio_t *pgpio = (void *)(CONFIG_SYS_MPC85xx_GPIO_ADDR);
	setbits_be32(&pgpio->gpdat, GPIO15);
	setbits_be32(&pgpio->gpdir, GPIO_DIR );
	clrsetbits_be32(&pgpio->gpdat, (GPIO07|GPIO13|GPIO14), (GPIO05|GPIO06|GPIO10));
}

int board_early_init_f(void)
{
	ccsr_gur_t *gur = (void *)(CONFIG_SYS_MPC85xx_GUTS_ADDR);

	setbits_be32(&(LBC_BASE_ADDR)->lbcr, CONFIG_SYS_LBC_LBCR);

	setbits_be32(&gur->pmuxcr,
			(MPC85xx_PMUXCR_SDHC_CD | MPC85xx_PMUXCR_SDHC_WP));
	clrbits_be32(&gur->sdhcdcr, SDHCDCR_CD_INV);

	clrbits_be32(&gur->pmuxcr, MPC85xx_PMUXCR_SD_DATA);
	setbits_be32(&gur->pmuxcr, MPC85xx_PMUXCR_TDM_ENA);

	board_gpio_init();
	board_cpld_init();

	return 0;
}

int misc_init_r(void)
{
	fsl_lbc_t *lbc = LBC_BASE_ADDR;

	/* UPM Table Configuration Code */
	static u32 UPMATable[] = 
	{
	    /* UPM Read Single RAM array entry */
		0xcfffcc00,  0x0ffcce00,  0x0ffccc00,  0x0ffcfc04, /*Words  0 to  3*/
		0x0fffdc00,  0xffffdc01,  0xfffffc00,  0xfffffc03, /*Words  4 to  7*/
        
		/* UPM Read Burst RAM array entry -> unused */
		0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc00, /*Words  8 to 11*/
		0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc00, /*Words 12 to 15*/
		0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc00, /*Words 16 to 19*/
		0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc01, /*Words 20 to 23*/

		/* UPM Write Single RAM array entry */
        0xcfffcc00,  0x00ffce00,  0x00ffdc00,  0x03ffcc00, /*Words 24 to 27*/
		0x3fffcc05,  0xfffffc00,  0xfffffc00,  0xfffffc01, /*Words 28 to 31*/

		/* UPM Write Burst RAM array entry -> unused */
        0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc00, /*Words 32 to 35*/
		0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc00, /*Words 36 to 39*/
		0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc00, /*Words 40 to 43*/
		0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc01, /*Words 44 to 47*/

		/* UPM Refresh Timer RAM array entry -> unused */
        0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc00, /*Words 48 to 51*/
		0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc00, /*Words 52 to 55*/
		0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc01, /*Words 56 to 59*/

		/* UPM Exception RAM array entry -> unused */
        0xfffffc00,  0xfffffc00,  0xfffffc00,  0xfffffc01  /*Words 60 to 63*/
	};

	upmconfig(UPMA, UPMATable, sizeof(UPMATable) / sizeof(UPMATable[0]));

	/* Set LUPWAIT to be active high and enabled */
	setbits_be32(&lbc->mamr, MxMR_UWPL | MxMR_GPL_x4DIS);

	puts("UPMA:  Configured for DSP HostDMA I/F\n");

	return 0;
}

/* P1024/15 POR device status register (GUTS_PORDEVSR) */
#define MPC85xx_PORDEVSR_ECW		0x80000000
#define MPC85xx_PORDEVSR_ECW_SHIFT	31
#define MPC85xx_PORDEVSR_ECP1		0x03000000
#define MPC85xx_PORDEVSR_ECP1_SHIFT	24
int checkboard(void)
{
	ccsr_gur_t *gur = (void *)(CONFIG_SYS_MPC85xx_GUTS_ADDR);
	ccsr_gpio_t *pgpio = (void *)(CONFIG_SYS_MPC85xx_GPIO_ADDR);
    u32 pordevsr = in_be32(&gur->pordevsr);
	u8 revison =  board_revision() + 'A';
	printf("Board: %s Rev %c\n", CONFIG_BOARDNAME, revison );

#ifdef CONFIG_PHYS_64BIT
	puts("(36-bit addrmap) ");
#endif
	cpld_check_rev();

	printf("TSEC1: %c%s Mode\n",
			(( pordevsr & MPC85xx_PORDEVSR_ECW ) >> MPC85xx_PORDEVSR_ECW_SHIFT ) ? ' ': 'R',
			(( pordevsr & MPC85xx_PORDEVSR_ECP1) >> MPC85xx_PORDEVSR_ECP1_SHIFT) == 2 ? "GMII": "MII");
	puts("SD/MMC: 4-bit Mode\n");

	/*DDR3, USB/ETH PHY out of reset*/
	udelay(10*1000);
	clrsetbits_be32(&pgpio->gpdat, GPIO15, GPIO07|GPIO13);

	return 0;
}

int board_early_init_r(void)
{
	const unsigned int flashbase = CONFIG_SYS_FLASH_BASE;
	const u8 flash_esel = find_tlb_idx((void *)flashbase, 1);

	/*
	 * Remap Boot flash region to caching-inhibited
	 * so that flash can be erased properly.
	 */

	/* Flush d-cache and invalidate i-cache of any FLASH data */
	flush_dcache();
	invalidate_icache();

	/* invalidate existing TLB entry for flash */
	disable_tlb(flash_esel);
#ifdef __LOW_FLASH
	set_tlb(1, flashbase, CONFIG_SYS_FLASH_BASE_PHYS,
			MAS3_SX|MAS3_SW|MAS3_SR, MAS2_I|MAS2_G,
			0, flash_esel, BOOKE_PAGESZ_16M, 1);
#else
	set_tlb(1, flashbase, CONFIG_SYS_FLASH_BASE_PHYS,
			MAS3_SX|MAS3_SW|MAS3_SR, MAS2_I|MAS2_G,
			0, flash_esel, BOOKE_PAGESZ_64M, 1);
	set_tlb(1, flashbase + 0x4000000, CONFIG_SYS_FLASH_BASE_PHYS + 0x4000000,
			MAS3_SX|MAS3_SW|MAS3_SR, MAS2_I|MAS2_G,
			0, flash_esel+1, BOOKE_PAGESZ_64M, 1);
#endif
	return 0;
}

int board_eth_init(bd_t *bis)
{
	struct tsec_info_struct tsec_info[4];
	ccsr_gpio_t *pgpio = (void *)(CONFIG_SYS_MPC85xx_GPIO_ADDR);
	int num = 0;

    if ( in_be32(&pgpio->gpdat) & GPIO12 ) {
		/* VOIPDB(IPLE) is mounted, CPU MDIO bus is disabled until the VOIPDB out of reset.*/
		setbits_be32(&pgpio->gpdat, GPIO14);
		/* wait VOIPDB CPLD device config - MIDO Bus enable */
		udelay(10*1000);
	}

#ifdef CONFIG_TSEC3
	/*TSEC1: no phy - TBI PHY ADDR: 1F*/
	tsec_t *regs = (tsec_t *)(TSEC_BASE_ADDR);
	out_be32(&regs->tbipa, CONFIG_SYS_TBIPA_VALUE);
	SET_STD_TSEC_INFO(tsec_info[num], 3);
	num++;
#endif
#ifdef CONFIG_TSEC1
	SET_STD_TSEC_INFO(tsec_info[num], 1);
	num++;
#endif
#ifdef CONFIG_TSEC2
	SET_STD_TSEC_INFO(tsec_info[num], 2);
	num++;
#endif

	if (!num) {
		printf("No TSECs initialized\n");
		return 0;
	}

	tsec_eth_init(bis, tsec_info, num);

	return pci_eth_init(bis);
}

extern u8 cpld_reconfig_status(void);
int last_stage_init(void)
{
	if ( cpld_reconfig_status() ) {
		/* fpga: reconfiguration error !! stop boot */
		setenv("bootcmd"	 , "");
	}
	return 0;
}

#ifdef CONFIG_OF_BOARD_SETUP
void ft_board_setup(void *blob, bd_t *bd)
{
	phys_addr_t base;
	phys_size_t size;

	ft_cpu_setup(blob, bd);

	base = getenv_bootm_low();
	size = getenv_bootm_size();

	fdt_fixup_memory(blob, (u64)base, (u64)size);

#if defined(CONFIG_PCI)
	FT_FSL_PCI_SETUP;
#endif

	fdt_fixup_dr_usb(blob, bd);
}
#endif

#if defined(CONFIG_SYS_DRAM_TEST)
static void ddr_check_ecc_memory_error( void )
{
	ccsr_ddr_t *ddr = (void *)CONFIG_SYS_MPC85xx_DDR_ADDR;
	if( in_be32(&ddr->sdram_cfg) & SDRAM_CFG_ECC_EN )
		printf("DDR3: Single-Bit ECC memory error %d\n", in_be32(&ddr->err_sbe) & 0xff);
}
int testdram(void)
{
	int i;
	u32 *p;
	struct dramlist_t {
        const char *type;
		u32 *top;
		u32 *end;
	} lists[2] = {
		{ "DDR3", (u32 *)CONFIG_SYS_MEMTEST_START, (u32 *)CONFIG_SYS_MEMTEST_END },
		{ "SRAM", (u32 *)CONFIG_SYS_SRAM_BASE	 , (u32 *)(CONFIG_SYS_SRAM_BASE+CONFIG_SYS_SRAM_SIZE-1) }
	};
	if ( !cpld_testmode() )
		return 0;

	for ( i=0; i<(sizeof(lists)/sizeof(lists[0])); i++ ) {
		printf("Testing %s from 0x%08x to 0x%08x\n",
			   lists[i].type,(u32)lists[i].top,(u32)lists[i].end);

		for (p = lists[i].top; p < lists[i].end; p++) {
			*p = 0xaaaaaaaa;
		}
		for (p = lists[i].top; p < lists[i].end; p++) {
			if (*p != 0xaaaaaaaa) {
				printf("%s phase 1 test fails at: 0x%p\n", lists[i].type,p);
				return 1;
			}
		}

		for (p = lists[i].top; p < lists[i].end; p++) {
			*p = 0x55555555;
		}
		for (p = lists[i].top; p < lists[i].end; p++) {
			if (*p != 0x55555555) {
				printf("%s phase 2 test fails at: 0x%p\n", lists[i].type,p);
				return 1;
			}
		}

		for (p = lists[i].top; p < lists[i].end; p++) {
#if defined(CONFIG_SYS_DDR_DATA_INIT)
			*p = CONFIG_SYS_DDR_DATA_INIT;
#else
			*p = 0;
#endif
		}
		printf("%s test passed.\n", lists[i].type);
	}
	ddr_check_ecc_memory_error();
	return 0;
}
#endif
#ifdef CONFIG_SHOW_ACTIVITY
void board_show_activity (ulong timebase)
{	/*timebase: decrement timer intr count*/
	if (!(timebase % (CONFIG_SYS_HZ / 10)))
		writeb( readb((CONFIG_SYS_CPLD_BASE+0x46)) ^ 0x08, (CONFIG_SYS_CPLD_BASE+0x46) );
}

void show_activity(int arg)
{
	switch(arg) {
	  case 0:/*main loop*/
		break;
	case 1:/*net*/
		break;
	}
}
#endif /* CONFIG_SHOW_ACTIVITY */
