/*
 * Copyright 2010-2011 NEC Infrontia Corp, Inc.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
  board_rev A: CPLD
        rev B: FPGA
 */

#include <common.h>
#include <asm/immap_85xx.h>
#include <asm/io.h>
#include <command.h>

extern u8 board_revision( void );

typedef struct cpld_data_cont{
	/*unicorn - cpld register */
					/*ind w/r	initial*/
	u16 dpll		;/*00 w		h'fff8 */
	u16 sflm_value	;/*02 w		h'fffe */
	u16 sflm_out	;/*04 w		h'fff0 */
	u16 load_sw		;/*06 r		h'fffx */
	u16 bwb_id		;/*08 r		h'fffx */
	u16 ms_sel		;/*0A rw	h'fffe */
	u16 hw_nrst		;/*0C rw	h'fffe */
	u16 res0		;/*0E -		- */

	u16 ccpu_id		;/*10 r		h'fffx */
	u16 slf_rst		;/*12 w		h'ffff */
	u16 dsp_spi_dn	;/*14 r		h'ffff */
	u16 hwy_sel		;/*16 w		h'ffff */
	u16 power		;/*18 rw	h'fffd */
	u16 spi_status	;/*1A r		- */
	u16 spi_req		;/*1C w		h'ff00 */
	u16 address_h	;/*1E w		h'fff0 */

	u16 address_l	;/*20 w		h'ff00 */
	u16 data		;/*22 rw	h'ff00 */
	u16 voip_ssi	;/*24 rw	h'ffff */
	u16 wdt_en		;/*26 w		h'fffe */
	u16 wdt_rst		;/*28 w		h'ffff */
	u16 load		;/*2A rw	h'fffe */
	u16 wdt_gen_cnt	;/*2C r		h'ffff */
	u16 res1[1]		;/*2E -		- */

	u16 res2[6]		;/*3x -		- */
	u16 rev_m		;/*3C r		h'fffx */
	u16 rev_s		;/*3E r		h'fffx */

	u16 gpio1		;/*40 r		h'fffc */
	u16 gpio2		;/*42 rw	h'fffe */
	u16 gpio3		;/*44 r		h'fff0 */
	u16 gpio4		;/*46 rw	h'fffx */
	u16 gpio5		;/*48 r		h'fffx */
#define DSW4	0x08	/*0:on, 1,off*/
#define DSW3	0x04
#define DSW2	0x02
#define DSW1	0x01
	u16 voip_hw_en	;/*4A rw	h'fff0 */
	u16 hwa_f_ts	;/*4C rw	h'ffff */
	u16 hwa_l_ts	;/*4E rw	h'ffff */

	u16 res3		;/*50 -		- */
	u16 rmii_sel	;/*52 rw	h'fffd */
	u16 alm_sts1	;/*54 r		h'ffxx */
	u16 almsts2		;/*56 r		h'ffxx */
	u16 act_sby		;/*58 rw	h'fffe */
	u16 mb_sw		;/*5A r		h'ffff */
	u16 alm_gen_w	;/*5C rw	h'ffff */
	u16 alm_gen_r	;/*5E r		h'ffff */
	u16 rtu_err		;/*60 r		h'ffff */
	u16 rtu_ctrl	;/*62 r/w	h'ffff */
	u16 rtu_info	;/*64 r		h'ffff */
	u16 res4[0x1a]	;
} cpld_data_t;

void cpld_watchdog_disable(void)
{
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	writeb( 0x00, &cpld_data->wdt_en );
}
void cpld_highway_reset(int active)
{
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	if( active )
		writeb( 0x00, &cpld_data->hw_nrst );
	else
		writeb( 0x01, &cpld_data->hw_nrst );
}
void cpld_clear_dpll(void)
{
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	writeb( 0x00, &cpld_data->dpll );
}

void cpld_poweroff_req(int off)
{
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	if( off )
		writeb( 0x00, &cpld_data->power );
	else
		writeb( 0x01, &cpld_data->power );
}

/*
 FPGA Reconfiguration
 */
DECLARE_GLOBAL_DATA_PTR;
#define FPGA_RCFG_WDTE	0x10	/* rtu_ctrl[4]    : watchdog timer enable */
#define FPGA_RCFG_START	0x8		/* rtu_ctrl[3]    : reconfig enable */
#define FPGA_RCFG_PAGE1	0x0		/* rtu_ctrl[2...0]: page select */
#define FPGA_RCFG_PAGE2	0x1
int cpld_reconfiguration(void)
{
	u32 voipdb_mode, voipdb_detect;
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	ccsr_gpio_t *pgpio	   = (void *)(CONFIG_SYS_MPC85xx_GPIO_ADDR);

	u8 reconf	  = FPGA_RCFG_START;
	if ( board_revision() == 0 )
		return 0;/*CPLD*/
	/* WS2: fpga */
	voipdb_detect= in_be32(&pgpio->gpdat) & 0x00080000;	/*GPIO12: 0 = none	 1 = detect*/
	voipdb_mode	 = in_be32(&pgpio->gpdat) & 0x00100000;	/*GPIO11: 0 = RGMII	 1 = RMII*/
	if ( (readb(&cpld_data->rev_m) & 0x0f) == 0x0f ) {
		if ( readb(&cpld_data->rtu_err) & 0x1f ) {
			/* reconfiguration error */
			return 0;
		}
		/* boot mode -> reconfig request */
		(void)get_clocks();
		gd->baudrate = CONFIG_BAUDRATE;
		serial_init ();
		console_init_f ();
		if ( voipdb_detect ) {
			reconf |= (!voipdb_mode) ? FPGA_RCFG_PAGE1: FPGA_RCFG_PAGE2;
		} else {
			reconf |= FPGA_RCFG_PAGE1;
		}
		printf("\nFPGA Boot[V%02x.%02x]: Reconfiguration Page %x \n", cpld_data->rev_m & 0x0f, cpld_data->rev_s & 0x0f, (reconf & 1) + 1 );
		udelay(5*1000);
		writeb( reconf, &cpld_data->rtu_ctrl );
		udelay(1000*1000);
		return 1;	/* wait cpld reconfig */
	} else {
		/* application mode */
		;
	}
	return 0;
}

void cpld_load_reg_set(u8 val)
{
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	writeb( val, &cpld_data->load );
}

unsigned char cpld_loadsw_reg_read(void)
{
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	if ( board_revision() == 1 )
		return !( readb(&cpld_data->load_sw) & 0x01 ); /* bugfixed: bit flip on target board rev B */
	else
		return ( readb(&cpld_data->load_sw) & 0x01 );
}

unsigned char cpld_testmode(void)
{
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	return !( readb(&cpld_data->gpio5) & DSW2 );
}

void board_cpld_init(void)
{
	cpld_reconfiguration();
	cpld_watchdog_disable();
	cpld_highway_reset(1);
	cpld_clear_dpll();
	cpld_poweroff_req(1);

	/* load switch check */
	if( cpld_loadsw_reg_read() )
		cpld_load_reg_set( 0 );	/* loadsw off */
	else
		cpld_load_reg_set( 1 );	/* loadsw on */
}
void cpld_check_rev(void)
{
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	printf("FPGA: V%02x.%02x\n", cpld_data->rev_m & 0x0f, cpld_data->rev_s & 0x0f);
}

u8 cpld_reconfig_status(void)
{
	cpld_data_t *cpld_data = (void *)(CONFIG_SYS_CPLD_BASE);
	u8 reconfig_error;
	if ( (readb(&cpld_data->rev_m) & 0x0f) != 0x0f ) {
		return 0;	/* normal page1/2 reconfig */
	}
	reconfig_error = readb(&cpld_data->rtu_err) & 0x1f;
	if ( reconfig_error ) {
		printf("*** FPGA Reconfiguration Error ***\n");
		/*
		 type[4...3]
		  b01  ... watchdog time out error
		  b10  ... nSTATUS error
		  b11  ... Configuration CRC error
		 page[2...0]
		  b001 ... page 1
		  b010 ... page 2
		 */
		printf("*** Error Type: %x ***\n", (reconfig_error & 0x18) >> 3);
		printf("*** Error Page: %x ***\n", (reconfig_error & 0x7 ));
	}
	return reconfig_error;
}
#if defined(CONFIG_CPLD_DEBUG)
static char* dtob(u8 x, char* buf, int size ) {
  int bit, i;
  buf[size] = '\0';
  for ( i = size - 1, bit = 1; i >= 0; i--, bit <<= 1 )
      buf[i] = (x & bit) ? '1': '0' ;
  return buf;
}
int do_cpld(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    char buf[8+1];
    const volatile cpld_data_t* p = (void *)(CONFIG_SYS_CPLD_BASE);
    printf( "CPLD BASE ADDR 0x%x\n", CONFIG_SYS_CPLD_BASE );    
	printf( "00: dpll        (w)%02x : %s\n", (u8)p->dpll        ,dtob((u8)p->dpll        , buf , 8));
	printf( " 2: sflm_value  (w)%02x : %s\n", (u8)p->sflm_value  ,dtob((u8)p->sflm_value  , buf , 8));
	printf( " 4: sflm_out    (w)%02x : %s\n", (u8)p->sflm_out    ,dtob((u8)p->sflm_out    , buf , 8));
	printf( " 6: load_sw        %02x : %s\n", (u8)p->load_sw     ,dtob((u8)p->load_sw     , buf , 8));
	printf( " 8: bwb_id         %02x : %s\n", (u8)p->bwb_id      ,dtob((u8)p->bwb_id      , buf , 8));
	printf( " A: ms_sel         %02x : %s\n", (u8)p->ms_sel      ,dtob((u8)p->ms_sel      , buf , 8));
	printf( " C: hw_nrst        %02x : %s\n", (u8)p->hw_nrst     ,dtob((u8)p->hw_nrst     , buf , 8));
	printf( "10: ccpu_id        %02x : %s\n", (u8)p->ccpu_id     ,dtob((u8)p->ccpu_id     , buf , 8));
	printf( " 2: slf_rst     (w)%02x : %s\n", (u8)p->slf_rst     ,dtob((u8)p->slf_rst     , buf , 8));
	printf( " 4: dsp_spi_dn     %02x : %s\n", (u8)p->dsp_spi_dn  ,dtob((u8)p->dsp_spi_dn  , buf , 8));
	printf( " 6: hwy_sel     (w)%02x : %s\n", (u8)p->hwy_sel     ,dtob((u8)p->hwy_sel     , buf , 8));
	printf( " 8: power          %02x : %s\n", (u8)p->power       ,dtob((u8)p->power       , buf , 8));
	printf( " A: spi_status     %02x : %s\n", (u8)p->spi_status  ,dtob((u8)p->spi_status  , buf , 8));
	printf( " C: spi_req     (w)%02x : %s\n", (u8)p->spi_req     ,dtob((u8)p->spi_req     , buf , 8));
	printf( " E: address_h   (w)%02x : %s\n", (u8)p->address_h   ,dtob((u8)p->address_h   , buf , 8));
	printf( "20: address_l   (w)%02x : %s\n", (u8)p->address_l   ,dtob((u8)p->address_l   , buf , 8));
	printf( " 2: data           %02x : %s\n", (u8)p->data        ,dtob((u8)p->data        , buf , 8));
	printf( " 4: voip_ssi       %02x : %s\n", (u8)p->voip_ssi    ,dtob((u8)p->voip_ssi    , buf , 8));
	printf( " 6: wdt_en      (w)%02x : %s\n", (u8)p->wdt_en      ,dtob((u8)p->wdt_en      , buf , 8));
	printf( " 8: wdt_rst     (w)%02x : %s\n", (u8)p->wdt_rst     ,dtob((u8)p->wdt_rst     , buf , 8));
	printf( " A: load           %02x : %s\n", (u8)p->load        ,dtob((u8)p->load        , buf , 8));
	printf( " C: wdt_gen_cnt    %02x : %s\n", (u8)p->wdt_gen_cnt ,dtob((u8)p->wdt_gen_cnt , buf , 8));
	printf( "3C: rev_m          %02x : %s\n", (u8)p->rev_m       ,dtob((u8)p->rev_m       , buf , 8));
	printf( " E: rev_s          %02x : %s\n", (u8)p->rev_s       ,dtob((u8)p->rev_s       , buf , 8));
	printf( "40: gpio1          %02x : %s\n", (u8)p->gpio1       ,dtob((u8)p->gpio1       , buf , 8));
	printf( " 2: gpio2          %02x : %s\n", (u8)p->gpio2       ,dtob((u8)p->gpio2       , buf , 8));
	printf( " 4: gpio3          %02x : %s\n", (u8)p->gpio3       ,dtob((u8)p->gpio3       , buf , 8));
	printf( " 6: gpio4          %02x : %s\n", (u8)p->gpio4       ,dtob((u8)p->gpio4       , buf , 8));
	printf( " 8: gpio5          %02x : %s\n", (u8)p->gpio5       ,dtob((u8)p->gpio5       , buf , 8));
	printf( " A: voip_hw_en     %02x : %s\n", (u8)p->voip_hw_en  ,dtob((u8)p->voip_hw_en  , buf , 8));
	printf( " C: hwa_f_ts       %02x : %s\n", (u8)p->hwa_f_ts    ,dtob((u8)p->hwa_f_ts    , buf , 8));
	printf( " E: hwa_l_ts       %02x : %s\n", (u8)p->hwa_l_ts    ,dtob((u8)p->hwa_l_ts    , buf , 8));
	printf( "52: rmii_sel       %02x : %s\n", (u8)p->rmii_sel    ,dtob((u8)p->rmii_sel    , buf , 8));
	printf( " 4: alm_sts1       %02x : %s\n", (u8)p->alm_sts1    ,dtob((u8)p->alm_sts1    , buf , 8));
	printf( " 6: almsts2        %02x : %s\n", (u8)p->almsts2     ,dtob((u8)p->almsts2     , buf , 8));
	printf( " 8: act_sby        %02x : %s\n", (u8)p->act_sby     ,dtob((u8)p->act_sby     , buf , 8));
	printf( " A: mb_sw          %02x : %s\n", (u8)p->mb_sw       ,dtob((u8)p->mb_sw       , buf , 8));
	printf( " C: alm_gen_w      %02x : %s\n", (u8)p->alm_gen_w   ,dtob((u8)p->alm_gen_w   , buf , 8));
	printf( " E: alm_gen_r      %02x : %s\n", (u8)p->alm_gen_r   ,dtob((u8)p->alm_gen_r   , buf , 8));
	/* boot mode */
	printf( "60: rtu_err		%02x : %s\n", (u8)p->rtu_err	 ,dtob((u8)p->rtu_err	  , buf , 8));
	printf( " 2: rtu_ctrl		%02x : %s\n", (u8)p->rtu_ctrl	 ,dtob((u8)p->rtu_ctrl	  , buf , 8));
	printf( " 4: rtu_info		%02x : %s\n", (u8)p->rtu_info	 ,dtob((u8)p->rtu_info	  , buf , 8));
	return 0;
}
U_BOOT_CMD(
	 cpld,	1,	1,	do_cpld,
	  "cpld register dump",
      ""
);
#endif

