/*
 * Copyright 2010-2011 NEC Infrontia Corp, Inc.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <netdev.h>
#include <malloc.h>
#include <linux/ctype.h>
#include <command.h>
#include <flash.h>

static int board_check_ethaddr(void)
{
	int i, cnt = 0;
	unsigned char *ethaddr;

    ethaddr = (unsigned char *)(CONFIG_MAC_ADDR + CONFIG_MAC_ADDR_OFFSET);

	/* MAC address check
	 * If ehtaddr[i] is all 0xff, MAC address is writen yet
	 */
	for (i = 0; i < 6; i++){
		if (ethaddr[i] == 0xff){
			cnt++;
		}
	}

	if (cnt >= 6){
		printf("Warning! MAC Address does not set!\n");
		return 1;
	}
	return 0;

}


static void board_get_enetaddr (int macaddr_idx,unsigned char * addr)
{
	int i = 0;
	unsigned char *ethaddr;

    ethaddr = (unsigned char *)(CONFIG_MAC_ADDR + CONFIG_MAC_ADDR_OFFSET + (6*macaddr_idx));

	for (i = 0; i < 6; i++){
		addr[i] = ethaddr[i];
	}
    printf("MAC%d address = %pM\n", macaddr_idx, addr);
	return ;
}

static int makeaddr(unsigned char *dst, unsigned char *src)
{
	char c;
	unsigned char *p;
	int i;

	p = src;
	for(i = 0 ; i < 12 ; i++ ){
		c = *p++;

		if( ( c >= '0' ) && ( c <= '9' ) ){
			c -= '0';
		}else if( ( c >= 'A' ) && ( c <= 'F' ) ){
			c = c - 'A' + 10;
		}else if( ( c >= 'a' ) && ( c <= 'f' ) ){
			c = c - 'a' + 10;
		}else{
			printf( "*** Invalid character [%x] detected.\n" , c );
			return 1;
		}
		if( i & 1 ){
			*dst++ |= c;
		}else{
			*dst = c << 4;
		}
	}
	return 0;
}

const char HexString[] = "0123456789ABCDEF";

static int gethexstring( char* buffer, size_t bufferSize )
{
	if( !buffer || ( bufferSize <= 0 ) )
		return 0;

	bufferSize--;
	int cursor = 0;
	for( ; ; ){
		int c = getc();
		c = toupper( c );
		switch( c ){
		  case 0x1b:	// ESC
			  puts( "* CANCEL *\n" );
			  return -1;
		  case '\n':
		  case '\r':
			  puts( "\n" );
			  buffer[cursor] = 0;
			  return cursor;
		  case '\b':
			  if( !cursor )
				  break;
			  puts( "\b \b" );
			  cursor--;
			  break;
		  default:
			  if( cursor < bufferSize ){
				  if( strchr( HexString , c ) ){
					  // Hex strng
					  printf( "%c" , c );
					  buffer[cursor++] = c;
				  }
			  }
			  break;
		}
	}
}

int do_setmaddr(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{

	int i, rc, skip, write_req, rcode = 0, index, result;
	unsigned long	flash_sect_addr, size;
	unsigned long	end_addr;
	unsigned char	macAddr1[6];
	char buff[6*2+1];

	char *tmp;
	const int ofsMagicNmber55 = 2;
	const int ofsMagicNmberAA = 1;
	const int ofsMagicVersion = 16;

	/* find last sector address */
	flash_sect_addr = CONFIG_MAC_ADDR;

	size = CONFIG_MAC_SECT_SIZE;
	end_addr = flash_sect_addr + size - 1;

    if ( argc >= 1 ) {
        board_get_enetaddr (0,macAddr1);
        board_get_enetaddr (1,macAddr1);
        if ( argc == 1 )
            return 0;
        switch (argv[1][0]) {
          case 'p':	/* program */
            break;
          case 'e':	/* erase */
            flash_sect_protect (0, flash_sect_addr, end_addr);
            puts ("Erasing Flash...");
            if ( flash_sect_erase (flash_sect_addr, end_addr) ) {
                return 1;
            }
            flash_sect_protect (1, flash_sect_addr, end_addr);
            return 0;
          case 'r':	/* read  */
          default:
            return 0;
        }
    }
	tmp = (char *)malloc(size);
	if (!tmp) {
		printf("ERROR memory allocate\n");
		return 1;
	}

	memcpy(tmp, (char *)flash_sect_addr, size);

	write_req = 0;
	/* eTSEC3:MAC0, eTSEC1:MAC1 */
	for (index = 0; index <= 1; index++) {
		skip = 0;
		for (;;){
			printf("Enter new MAC%d addrss (ESC=skip) : ", index);
			result = gethexstring( buff , sizeof( buff ) );
			if( result < 0 ){
				skip = 1;
				break;
			}
			if( result == (6*2) )
				break;
			printf( "* ERROR: MAC%d address length error\n", index );
            free(tmp);
			return 1;
		}
		if (!skip) {
			if (makeaddr((unsigned char*)macAddr1, (unsigned char*)buff)) {
				free(tmp);
				return 1;
			}
			printf("Input Ether Address %x:%x:%x:%x:%x:%x\n",
				   macAddr1[0], macAddr1[1], macAddr1[2],
				   macAddr1[3], macAddr1[4], macAddr1[5]);
			for (i = 0; i < 6; i++) {
				tmp[CONFIG_MAC_ADDR_OFFSET + (sizeof(macAddr1)*index) + i] = macAddr1[i];
			}
			write_req = 1;
		}
	}

	if (!write_req) {
		free(tmp);
		return 0;
	}

	tmp[size - ofsMagicNmber55] = 0x55;
	tmp[size - ofsMagicNmberAA] = 0xaa;
	*(int *)(tmp + size - ofsMagicVersion) = 0;


	(void) flash_sect_protect (0, flash_sect_addr, end_addr);

	puts ("Erasing Flash...");
	if (flash_sect_erase (flash_sect_addr, end_addr)) {
		free(tmp);
		return 1;
	}
	puts ("Writing to Flash... ");
	rc = flash_write(tmp, flash_sect_addr, size);
	if (rc != 0) {
		flash_perror (rc);
		rcode = 1;
	} else {
		puts ("done\n");
	}

	/* try to re-protect */
	(void) flash_sect_protect (1, flash_sect_addr, end_addr);

	free(tmp);
	return rcode;
}

int	mac_read_from_flashsector(void)
{
	unsigned char enetaddr[6];

	if (board_check_ethaddr()) {
		do_setmaddr(NULL, 0, 0, NULL);
	}

	board_get_enetaddr(0,enetaddr);
	eth_setenv_enetaddr("ethaddr", enetaddr);

	board_get_enetaddr(1,enetaddr);
	eth_setenv_enetaddr("eth1addr", enetaddr);

	return 0;
}

#ifndef CONFIG_CMD_EEPROM
U_BOOT_CMD(
	 mac,	2,	1,	do_setmaddr,
	  "program the MAC addresses in Flash",
      "[read|erase|program] program the MAC address in Flash"
);
#endif
